#!/bin/sh

main(){
    # Please adapt the paths of the tools if needed
    READEMPTION_BIN_PATH=reademption
    FASTQ_DUMP_BIN_PATH=fastq-dump
    FASTQ_QUALITLY_TRIMMER_BIN_PATH=fastq_quality_trimmer
    FASTQ_TO_FASTA_BIN_PATH=fastq_to_fasta

    READEMPTION_FOLDER=READemption_analysis
    REF_SEQ_FOLDER=$READEMPTION_FOLDER/input/reference_sequences/
    ANNOTATION_FOLDER=$READEMPTION_FOLDER/input/annotations
    READ_LIB_FOLDER=$READEMPTION_FOLDER/input/reads
    SRA_FOLDER=SRA
    MAPPING_PROCESSES=40
    COVERAGE_PROCESSES=40
    GENE_QUANTI_PROCESSES=40
    NCBI_SOURCE_FTP=ftp://ftp.ncbi.nih.gov/genomes/Bacteria/Stenotrophomonas_maltophilia_K279a_uid61647/
    
    create_folders
    get_genome_fasta_files
    get_gff_files
    get_sra_files
    convert_sra_and_trim
    run_read_alignment
    build_coverage_files
    run_gene_quanti
    run_deseq_analysis
    run_viz_align
    run_viz_gene_quanti
    run_viz_deseq
}

create_folders(){
    mkdir -p $SRA_FOLDER
    $READEMPTION_BIN_PATH create $READEMPTION_FOLDER
}

get_genome_fasta_files(){
    wget -cP $REF_SEQ_FOLDER \
	$NCBI_SOURCE_FTP/*fna
    # Fix headers
    for FASTA_FILE in $(ls ${REF_SEQ_FOLDER} | grep fna$)
    do
        ACCESSION=$(head -n 1 ${REF_SEQ_FOLDER}/${FASTA_FILE} | cut -f 4 -d"|")
        cat ${REF_SEQ_FOLDER}/${FASTA_FILE} | sed "s/>/>${ACCESSION} /" \
            > ${REF_SEQ_FOLDER}/$(basename ${FASTA_FILE} .fna).fa \
            && rm ${REF_SEQ_FOLDER}/${FASTA_FILE}
    done
}

get_gff_files(){
    wget -cP $READEMPTION_FOLDER/input/annotations/ \
	$NCBI_SOURCE_FTP/*gff
}

get_sra_files(){
    wget -cP $SRA_FOLDER/ \
	ftp://ftp-trace.ncbi.nlm.nih.gov/sra/sra-instant/reads/ByExp/sra/SRX%2FSRX113%2FSRX1133363/SRR2145274/SRR2145274.sra \
	ftp://ftp-trace.ncbi.nlm.nih.gov/sra/sra-instant/reads/ByExp/sra/SRX%2FSRX113%2FSRX1133364/SRR2145275/SRR2145275.sra \
	ftp://ftp-trace.ncbi.nlm.nih.gov/sra/sra-instant/reads/ByExp/sra/SRX%2FSRX113%2FSRX1133365/SRR2145276/SRR2145276.sra \
	ftp://ftp-trace.ncbi.nlm.nih.gov/sra/sra-instant/reads/ByExp/sra/SRX%2FSRX113%2FSRX1133366/SRR2145277/SRR2145277.sra \
	ftp://ftp-trace.ncbi.nlm.nih.gov/sra/sra-instant/reads/ByExp/sra/SRX%2FSRX113%2FSRX1133367/SRR2145278/SRR2145278.sra \
	ftp://ftp-trace.ncbi.nlm.nih.gov/sra/sra-instant/reads/ByExp/sra/SRX%2FSRX113%2FSRX1133368/SRR2145279/SRR2145279.sra
}

convert_sra_and_trim(){
    for SRA_ID_AND_NAME in \
	SRR2145274:K279a_small_colonies_Replicate_1 \
	SRR2145275:K279a_small_colonies_Replicate_2 \
	SRR2145276:K279a_big_colonies_Replicate_1 \
	SRR2145277:K279a_big_colonies_Replicate_2 \
	SRR2145278:K279a_without_ampicillin_Replicate_1 \
	SRR2145279:K279a_without_ampicillin_Replicate_2
    do
	SRA_ID=$(echo $SRA_ID_AND_NAME | cut -f 1 -d:)
	NAME=$(echo $SRA_ID_AND_NAME | cut -f 2 -d:)
	$FASTQ_DUMP_BIN_PATH  \
	    --stdout \
	    $SRA_FOLDER/${SRA_ID}.sra \
	    | $FASTQ_QUALITLY_TRIMMER_BIN_PATH \
	    -Q33 \
	    -t 20 \
	    -l 1 \
	    | $FASTQ_TO_FASTA_BIN_PATH \
	    -Q33 \
	    -o $READ_LIB_FOLDER/${NAME}.fa &
    done
    wait
}

run_read_alignment(){
    $READEMPTION_BIN_PATH \
        align \
	-r \
        -p $MAPPING_PROCESSES \
        -a 95 \
        -l 12 \
	--poly_a_clipping \
	--progress \
        $READEMPTION_FOLDER
}

build_coverage_files(){
    $PYTHON_PATH $READEMPTION_BIN_PATH \
        coverage \
        -p $COVERAGE_PROCESSES \
        $READEMPTION_FOLDER
}

run_gene_quanti(){
    $PYTHON_PATH $READEMPTION_BIN_PATH \
	gene_quanti \
	-p $GENE_QUANTI_PROCESSES \
	--features CDS,rRNA,tRNA \
	$READEMPTION_FOLDER
}

run_deseq_analysis(){
    $PYTHON_PATH $READEMPTION_BIN_PATH \
	deseq \
	-l K279a_big_colonies_Replicate_1.fa,K279a_big_colonies_Replicate_2.fa,K279a_small_colonies_Replicate_1.fa,K279a_small_colonies_Replicate_2.fa,K279a_without_ampicillin_Replicate_1.fa,K279a_without_ampicillin_Replicate_2.fa \
	-c K279a_big_colonies,K279a_big_colonies,K279a_small_colonies,K279a_small_colonies,K279a_without_ampicillin,K279a_without_ampicillin \
	$READEMPTION_FOLDER
}

run_viz_align(){
   $PYTHON_PATH $READEMPTION_BIN_PATH \
        viz_align \
        $READEMPTION_FOLDER
}

run_viz_gene_quanti(){
   $PYTHON_PATH $READEMPTION_BIN_PATH \
        viz_gene_quanti \
        $READEMPTION_FOLDER
}

run_viz_deseq(){
   $PYTHON_PATH $READEMPTION_BIN_PATH \
        viz_deseq \
        $READEMPTION_FOLDER
}

main
